#pragma once
#include <stdint.h>

#ifndef CPU_TEST
#include "freertos/FreeRTOS.h"
#include "freertos/queue.h"
#else
#include "cpu_test_harness.h"
#endif

#define INFO_GROUP_LABEL_MAX_SIZE 12lu
typedef struct {
    char label[INFO_GROUP_LABEL_MAX_SIZE];
    char* time_str;
    union {
        uint32_t num;
        char* str;
    } data;
    enum {
        GROUP_DATA_NUM,
        GROUP_DATA_STR
    } data_type;
} InfoGroup;

#define INFO_FRAME_MAX_GROUPS 12lu
typedef struct {
    InfoGroup groups[INFO_FRAME_MAX_GROUPS];
    uint8_t group_count;
} InfoFrame;

typedef QueueHandle_t InfoFrameQueue;

typedef enum {
    GFS_NotStarted,
    GFS_Tag,
    GFS_Data,
    GFS_Checksum,
    GFS_Done,
    GFS_Invalid
} GroupFillStep;

#define POSTPONE_BUFFER_MAXSIZE 32lu
typedef struct {
    InfoFrameQueue frame_queue;
    InfoFrame* cur_frame;
    char checksum;
    GroupFillStep fill_step;
    char postpone_buffer[POSTPONE_BUFFER_MAXSIZE];
    size_t postpone_buffer_size;
} DecodeState;

/// Initialize a decode state
DecodeState* create_decode_state(size_t queue_size);
/// Frees a DecodeState
void free_decode_state(DecodeState*);

/// Deletes the contents of an InfoGroup
void tic_delete_group(InfoGroup* group);
/** Deletes the contents of an InfoFrame. The call to free() itself must be
 * done by the user */
void tic_delete_frame(InfoFrame* frame);

/**
 * Discard characters until just before a frame is started, and return a
 * pointer to the beginning of the next frame (including the start of frame
 * marker), or NULL if no frame was started.
 */
char* tic_discard_until_frame(char* buffer, size_t buf_size);

/**
 * Decode the buffer, updating the decode state and emitting frames as they are
 * finished.
 * Returns how many groups were decoded.
 */
int tic_decode(char* buffer, size_t buf_size, DecodeState* state);
