/* UART reception file -- reads data from Linky's TIC */
#include <stdio.h>
#include "driver/uart.h"
#include "driver/gpio.h"
#include "esp_sleep.h"
#include "esp_log.h"

#include "tic.h"

/**
 * This is an example which echos any data it receives on configured UART back to the sender,
 * with hardware flow control turned off. It does not use UART driver event queue.
 *
 * - Port: configured UART
 * - Receive (Rx) buffer: on
 * - Transmit (Tx) buffer: off
 * - Flow control: off
 * - Event queue: off
 * - Pin assignment: see defines below (See Kconfig)
 */

#define TIC_RXD (CONFIG_TIC_UART_RXD)
#define TIC_RTS (UART_PIN_NO_CHANGE)
#define TIC_CTS (UART_PIN_NO_CHANGE)

#define TIC_UART_PORT_NUM      (CONFIG_TIC_UART_PORT_NUM)
#define TIC_UART_BAUD_RATE     (CONFIG_TIC_UART_BAUD_RATE)
static const char *TAG = "TIC UART";

#define BUF_SIZE (1024)
#define TIC_QUEUE_SIZE 64

static void uart_light_sleep() {
    // TODO
    /*
    ESP_LOGI(TAG, "Entering sleep.\n");
    vTaskDelay(10/portTICK_PERIOD_MS);
    ESP_ERROR_CHECK(esp_light_sleep_start());
    esp_sleep_wakeup_cause_t wakeup_cause = esp_sleep_get_wakeup_cause();
    ESP_LOGI(TAG, "Woken up: %d\n", wakeup_cause);
    */
}

void uart_listen_task(void *arg)
{
    /* Configure parameters of an UART driver,
     * communication pins and install the driver */
    uart_config_t uart_config = {
        .baud_rate = TIC_UART_BAUD_RATE,
        .data_bits = UART_DATA_8_BITS,
        .parity    = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_1,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE,
        .source_clk = UART_SCLK_DEFAULT,
    };
    int intr_alloc_flags = 0;

#if CONFIG_UART_ISR_IN_IRAM
    intr_alloc_flags = ESP_INTR_FLAG_IRAM;
#endif

    ESP_ERROR_CHECK(uart_param_config(TIC_UART_PORT_NUM, &uart_config));
    ESP_ERROR_CHECK(uart_set_pin(TIC_UART_PORT_NUM, UART_PIN_NO_CHANGE, TIC_RXD, TIC_RTS, TIC_CTS));
    ESP_ERROR_CHECK(uart_driver_install(TIC_UART_PORT_NUM, BUF_SIZE * 2, 0, 0, NULL, intr_alloc_flags));
    //ESP_ERROR_CHECK(uart_set_wakeup_threshold(TIC_UART_PORT_NUM, 3));
    //ESP_ERROR_CHECK(esp_sleep_enable_uart_wakeup(TIC_UART_PORT_NUM));

    // Configure a temporary buffer for the incoming data
    char *data = (char *) malloc(BUF_SIZE);

    // Setup the TIC decoding
    DecodeState* tic_state = create_decode_state(TIC_QUEUE_SIZE);
    int initialized = 0;

    // Discard any data already buffered
    ESP_ERROR_CHECK(uart_flush(TIC_UART_PORT_NUM));

    ESP_LOGI(TAG, "Start receiving.\n");
    while (1) {
        int len = uart_read_bytes(TIC_UART_PORT_NUM, data, (BUF_SIZE - 1), 30 / portTICK_PERIOD_MS);
        if (len) {
            if(!initialized) {
                char* init_begin = tic_discard_until_frame(data, len);
                if(init_begin != NULL) {
                    size_t discarded = init_begin - data;
                    initialized = 1;
                    tic_decode(init_begin, len - discarded, tic_state);
                }
            } else {
                tic_decode(data, len, tic_state);
            }

            data[len] = '\0';
            ESP_LOGI(TAG, "Recv str: %s", (char *) data);
        }
        else {
            uart_light_sleep();
        }
    }

    free_decode_state(tic_state);
}
